import time
import unittest

from vika import Vika

from . import TEST_API_BASE, TEST_API_TOKEN, TEST_TABLE


class TestUpdateRecords(unittest.TestCase):
    def setUp(self):
        vika = Vika(TEST_API_TOKEN)
        vika.set_api_base(TEST_API_BASE)
        self.dst = vika.datasheet(TEST_TABLE)

    def test_record_update(self):
        # 更新单个字段
        record = self.dst.records.get(title="无人生还")
        record.title = "无人生还2"
        self.assertEqual(record.title, "无人生还2")
        time.sleep(1)

        # 更新多个字段
        record = self.dst.records.get(title="无人生还2")
        r = record.update({
            "title": '无人生还3',
            "comment": '真好看'
        })
        self.assertEqual(r.title, "无人生还3")
        self.assertEqual(r.comment, "真好看")
        time.sleep(1)

        # 更新多条记录
        self.dst.records.filter(
            title="无人生还3").update(title="无人生还4")
        record = self.dst.records.get(title="无人生还4")
        self.assertEqual(record.title, "无人生还4")
        time.sleep(1)

    def tearDown(self):
        self.dst.records.filter(title="无人生还4").update(title="无人生还")


if __name__ == '__main__':
    unittest.main()



from typing import List

from vika.types import MetaView


class ViewManager:
    def __init__(self, dst: 'Datasheet'):
        self.dst = dst

    def all(self) -> List[MetaView]:
        """
        查询当前维格表所有视图
        @return: List[MetaView]
        """
        return self.dst.get_views()



from typing import Dict, Any
from typing import TypeVar, Generic
from urllib.parse import urlparse

T = TypeVar('T')

FieldKeyMap = Dict[str, str]


def chunks(_list, chunk_size):
    for i in range(0, len(_list), chunk_size):
        yield _list[i: i + chunk_size]


def trans_key(field_key_map: FieldKeyMap, key: str):
    """
    存在字段映射时，将映射的 key 转为实际的 key
    """
    if key in ["_id", "recordId"]:
        return key
    if field_key_map:
        _key = field_key_map.get(key, key)
        return _key
    return key


def trans_data(field_key_map: FieldKeyMap, data: Dict[str, Any]):
    """
    配置字段映射关系后，将传入的 record 数据转化为 API 理解的数据
    """
    if field_key_map:
        _data = {}
        for k, v in data.items():
            _k = field_key_map.get(k, k)
            _data[_k] = v
        return _data
    return data


def query_parse(field_key_map: FieldKeyMap, **kwargs):
    """
    将查询条件转化为 filterByFormula
    records.filter(title="hello", subtitle="world") => '{title}="hello" AND {subtitle}="world"'
    1. 通过 filter 和 get 参数查询到的只能转化为 and 条件。
    """
    query_str = ""
    for k, v in kwargs.items():
        if query_str:
            query_str += " AND "
        # 处理空值
        if v is None:
            v = 'BLANK()'
        # 处理字符串
        if isinstance(v, str):
            v = f'"{v}"'
        # 处理数组类型的值，多选，成员？
        if isinstance(v, list):
            v = f'"{", ".join(v)}"'
        query_str += f'{{{trans_key(field_key_map, k)}}}={v}'
    return query_str


def handle_response(r, resp_class: Generic[T]) -> T:
    if r["success"]:
        r = resp_class(**r)
        return r
    raise Exception(r['message'])


def check_sort_params(sort):
    if not isinstance(sort, list):
        return False
    return all([('field' in i and 'order' in i) for i in sort])


def get_dst_id(dst_id_or_url: str):
    if dst_id_or_url.startswith("dst"):
        return dst_id_or_url
    elif dst_id_or_url.startswith("http"):
        url = urlparse(dst_id_or_url)
        url_path_list = url.path.split("/")
        dst_id = url_path_list[-2]
        return dst_id
    else:
        raise Exception('Bad Datasheet Id or URL')



from vika.node import NodeManager


class Space:
    def __init__(self, vika: "Vika", space_id: str):
        self.vika = vika
        self.id = space_id

    @property
    def nodes(self):
        return NodeManager(self.vika, space_id=self.id)



import unittest
import time
from vika import Vika
from . import TEST_TABLE, TEST_API_BASE, TEST_API_TOKEN


class TestCreateRecords(unittest.TestCase):
    def setUp(self):
        vika = Vika(TEST_API_TOKEN)
        vika.set_api_base(TEST_API_BASE)
        self.dst = vika.datasheet(TEST_TABLE)

    def test_record_create(self):
        time.sleep(1)
        record = self.dst.records.create({
            "title": "高等数学"
        })
        time.sleep(1)
        self.assertIsNotNone(record._id)
        records = self.dst.records.bulk_create([
            {
                "title": "离散数学"
            },
            {
                "title": "线性代数"
            }
        ])
        self.created_records = records + [record]
        for rec in records:
            self.assertIsNotNone(rec._id)

    def tearDown(self):
        self.dst.delete_records(self.created_records)


if __name__ == '__main__':
    unittest.main()



import unittest

from vika import Vika
from . import TEST_TABLE, TEST_API_BASE, TEST_API_TOKEN


class TestGetViews(unittest.TestCase):
    def setUp(self):
        vika = Vika(TEST_API_TOKEN)
        vika.set_api_base(TEST_API_BASE)
        self.dst = vika.datasheet(TEST_TABLE)

    def test_views_all(self):
        views = self.dst.views.all()
        self.assertIsInstance(views, list)
        first_view = views[0]
        self.assertEqual(first_view.name, '维格视图')
        self.assertEqual(first_view.type, 'Grid')


if __name__ == '__main__':
    unittest.main()



from urllib.parse import urljoin

from vika.types.response import (
    GETSpaceListResponse
)
from vika.utils import handle_response


class SpaceManager:
    def __init__(self, vika: 'Vika', **kwargs):
        self.vika = vika

    @property
    def _api_endpoint(self):
        return urljoin(self.vika.api_base, f"/fusion/v1/spaces")

    def _get_spaces(self) -> GETSpaceListResponse:
        """
        获取 field meta
        """
        r = self.vika.request.get(self._api_endpoint).json()
        return handle_response(r, GETSpaceListResponse)

    def all(self):
        """
        获取当前用户的空间站列表
        @return: List[SpaceListItem]
        """
        spaces_resp = self._get_spaces()
        return spaces_resp.data.spaces

    def get(self, space_id: str):
        """
        获取指定空间站信息
        @param space_id: 空间站 ID
        @return:
        """
        pass



from typing import Optional, List
from vika.types.field import MetaField
from vika.utils import trans_key


class FieldManager:
    def __init__(self, dst: 'Datasheet'):
        self.dst = dst
        self._is_fetched = False

        # 获取的 fields 信息全部的获取一次就缓存起来
        self._fields = []
        # 辅助数据
        self._meta_field_id_map = {}
        self._meta_field_name_map = {}

    def _check_meta(self, **kwargs):
        if not self._is_fetched:
            fields_resp = self.dst.get_fields(**kwargs)
            self._set_meta_fields(fields_resp.data.items)
            self._is_fetched = True

    def _set_meta_fields(self, fields: List[MetaField]):
        self._fields = fields
        for field in fields:
            self._meta_field_id_map[field.id] = field
            self._meta_field_name_map[field.name] = field

    def __contains__(self, item):
        """
        field_key 是否在 dst.fields 中
        @param item: field_key，字段名
        @return: bool
        """
        self._check_meta()
        if isinstance(item, MetaField):
            return item in self._fields
        field_key = trans_key(self.dst.field_key_map, item)
        return field_key in self._meta_field_id_map or field_key in self._meta_field_name_map

    def all(self, **kwargs) -> List[MetaField]:
        """
        获取当前维格表的全部字段信息
        @param kwargs:
            - viewId: 'viewId' 获取指定视图的字段信息，视图下隐藏的字段不会返回。
        @return:
        """
        # 获取指定视图的字段信息，一次性的不缓存。其实也可以缓存下来？
        if kwargs:
            fields_resp_via_view_id = self.dst.get_fields(**kwargs)
            return fields_resp_via_view_id.data.items
        self._check_meta()
        return self._fields

    def get(self, field_key: str, **kwargs) -> Optional[MetaField]:
        """
        通过 field_key 获取, 维格表内字段不能重名， id 和 name 都是唯一标识
        dst.fields.get('fldxxxxxxxx')
        dst.fields.get('title')
        @param field_key: 字段标识
        @param kwargs:
        @return: MetaField
        """
        field_key = trans_key(self.dst.field_key_map, field_key)
        # 没获取过 meta 先请求 meta
        self._check_meta()
        # 获取过 meta 直接返回字段
        if self.dst.field_key == "id":
            return self._meta_field_id_map.get(field_key, None)
        if self.dst.field_key == "name":
            return self._meta_field_name_map.get(field_key, None)
        # 找不到字段
        return None

    def __getitem__(self, index):
        self._check_meta()
        return self._fields[index]



import os
import time
import unittest

from vika import Vika

from . import TEST_API_BASE, TEST_API_TOKEN, TEST_TABLE


class TestUploadFile(unittest.TestCase):
    def setUp(self):
        self.vika = Vika(TEST_API_TOKEN)
        self.vika.set_api_base(TEST_API_BASE)
        self.dst = self.vika.datasheet(TEST_TABLE)

    def test_upload_file(self):
        test_url = "https://img9.doubanio.com/view/subject/s/public/s26849345.jpg"
        test_file = self.dst.upload_attachment(test_url)
        self.assertIsNotNone(test_file.get("token"))

        time.sleep(2)

        test_local_file = "vikaji.png"
        filepath = os.path.join(os.path.dirname(__file__), test_local_file)
        test_file = self.dst.upload_file(filepath)
        self.assertEqual(test_file.get("mimeType"), "image/png")


if __name__ == "__main__":
    unittest.main()
